program test_set_max_speed

  ! Libraries:
  use contour_531, only: convert_to_ind
  use jumble, only: get_command_arg_dyn

  use config_m, only: config
  use derived_types, only: eddy, shpc_slice_handler, ssh_contour
  use input_ssh_m, only: input_ssh
  use read_eddy_m, only: read_eddy
  use set_max_speed_m, only: set_max_speed
  use shpc_close_m, only: shpc_close
  use shpc_create_m, only: shpc_create
  use shpc_open_m, only: shpc_open
  use write_eddy_m, only: write_eddy

  implicit none

  logical cyclone
  logical periodic ! grid is periodic in longitude
  integer nlon, nlat, d, i
  real, allocatable:: ssh(:, :) ! (nlon, nlat) sea-surface height, in m
  real, allocatable:: u(:, :), v(:, :) ! (nlon, nlat) wind, in m s-1
  type(eddy) e
  real corner(2), step(2) ! in rad
  TYPE(shpc_slice_handler) hshp
  character(len = :), allocatable:: shpc_dir

  ! Window around the extremum:

  integer llc(2) ! indices in global grid of lower left corner
  integer urc(2) ! indices in global grid of upper right corner

  real corner_window(2) ! longitude and latitude of the window around
  ! the extremum, in rad

  type(ssh_contour) cont_list(1)
  ! Contour list. The outermost contour is element with subscript
  ! n_cont = 1. The contours are in monotonic order of ssh.

  !----------------------------------------------------------------

  print *, "test_set_max_speed: cyclone = ? "
  read *, cyclone
  call config
  call input_ssh(corner, step, periodic, ssh, u, v)
  print *, "test_set_max_speed: Reading from shapefiles..."
  call get_command_arg_dyn(3, shpc_dir, "Required 3rd argument: SHPC-directory")
  call shpc_open(hshp, shpc_dir, cyclone, slice = 0, pszaccess = "rb")
  call read_eddy(e, d, i, hshp, ishape = 0)
  CALL shpc_close(hshp)
  if (.not. periodic) nlon = size(ssh, 1)
  nlat = size(ssh, 2)

  if (e%out_cont%closed) then
     ! Restrict the field to the outermost contour:

     llc = floor(convert_to_ind(minval(e%out_cont%points, dim = 2), &
          corner, step))

     urc = ceiling(convert_to_ind(maxval(e%out_cont%points, dim = 2), &
          corner, step))

     ! Should have no effect except because of roundup error:
     urc(2) = min(urc(2), nlat)
     if (.not. periodic) urc(1) = min(urc(1), nlon)

     corner_window = corner + (llc - 1) * step
     e%extr%coord_proj = nint((e%extr%coord - corner) / step) + 1
     cont_list(1) = e%out_cont
     call set_max_speed(e, e%extr%coord_proj - llc + 1, &
          ssh(llc(1):urc(1), llc(2):urc(2)), &
          u(llc(1):urc(1), llc(2):urc(2)), &
          v(llc(1):urc(1), llc(2):urc(2)), corner_window, step, cont_list)
     call shpc_create(hshp, shpc_dir = "SHPC", cyclone = cyclone, slice = 0, &
          grid_lon_lat = .true.)
     call write_eddy(e, hshp, d, i)
     write(hshp%unit, fmt = *) 0
     CALL shpc_close(hshp)
     print *, 'test_set_max_speed: Created "SHPC".'
  else
     print *, "No outermost contour found."
  end if

end program test_set_max_speed
